==> ogdl-src.0.9.jar: shohaku/ogdl/OgdlParser.java <==
/*
 * shohaku
 * Copyright (C) 2006  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.ogdl;

import java.util.Collection;
import java.util.List;
import java.util.Map;

/**
 * OGDLの構文解析機能を提供します。<br>
 * このパーザが個別機能を提供する、他のパーザの入口として機能します。
 */
class OgdlParser {

    /*
     * Prefix Expression
     */

    static Object evaluateExpression(OgdlEvent ev) {
        final String ptn = ev.ptn;
        final OgdlParseIndex off = ev.off;
        final char open = ptn.charAt(off.get());
        final int begin = off.get();

        switch (open) {
        case '!':
            // Logical Complement Operator
            return OgdlUnaryOperatorParser.evaluateLogicalComplement(ev);
            // case '"':
            // case '#':
            // case '$':
        case '%':
            // Regex
            return OgdlPrimitiveParser.evaluateRegexPattern(ev);
            // case '&':
        case '\'':
            // String
            return OgdlPrimitiveParser.evaluateEncloseString(ev);
        case '(':
            // Operator Function or Expressions Group
            return OgdlOperatorParser.evaluateEncloseOperator(ev);
            // case ')':
            // case '*':
            // case '+':
            // case ',':
        case '-':
            // Unary Minus Operator
            return OgdlUnaryOperatorParser.evaluateUnaryMinus(ev);
            // case '.':
        case '/':
            // Date Time
            return OgdlPrimitiveParser.evaluateDateTime(ev);
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
            // Number
            return OgdlPrimitiveParser.evaluateNumber(ev);
            // case ':':
            // case ';':
        case '<':
            // Set
            return OgdlCollectionParser.evaluateDefaultSet(ev);
            // case '=':
            // case '>':
            // case '?':
        case '@':
            // Reference or Class Operator
            if (isLiteral(ev, "@@", ptn, begin)) {
                // Class Operator
                return OgdlPrimitiveParser.evaluateClassByExtendName(ev);
            }
            // Reference Operator
            ev.off.increment();
            return OgdlPrimitiveParser.evaluateReference(ev);
            // case 'A':
            // case 'B':
            // case 'C':
        case 'D':
            // Date Time
            ev.off.increment();
            return OgdlPrimitiveParser.evaluateDateTime(ev);
            // case 'E':
        case 'F':
            // Operator Function or Expressions Group
            ev.off.increment();
            return OgdlOperatorParser.evaluateOperator(ev);
            // case 'G':
            // case 'H':
        case 'I':
            // Infinity
            if (isLiteral(ev, "Infinity", ptn, begin)) {
                return OgdlPrimitiveParser.evaluateInfinity(ev);
            }
            break;
        // case 'J':
        // case 'K':
        // case 'L':
        // case 'M':
        case 'N':
            // NaN
            if (isLiteral(ev, "NaN", ptn, begin)) {
                return OgdlPrimitiveParser.evaluateNaN(ev);
            }
            break;
        // case 'O':
        // case 'P':
        // case 'Q':
        case 'R':
            // Regex
            ev.off.increment();
            return OgdlPrimitiveParser.evaluateRegexPattern(ev);
        case 'S':
            // String
            ev.off.increment();
            return OgdlPrimitiveParser.evaluateString(ev);
            // case 'T':
        case 'U':
            // Character
            ev.off.increment();
            return OgdlPrimitiveParser.evaluateCharacter(ev);
            // case 'V':
            // case 'W':
            // case 'X':
            // case 'Y':
            // case 'Z':
        case '[':
            // List
            return OgdlCollectionParser.evaluateDefaultList(ev);
            // case '\\':
            // case ']':
            // case '^':
            // case '_':
        case '`':
            // Character
            return OgdlPrimitiveParser.evaluateCharacter(ev);
        case 'a':
            // array
            if (isPrefixName(ev, "array:", ptn, begin)) {
                return OgdlCollectionParser.evaluateArray(ev);
            }
            break;
        // case 'b':
        case 'c':
            // class
            if (isPrefixName(ev, "class:", ptn, begin)) {
                return OgdlPrimitiveParser.evaluateClassByExtendName(ev);
            }
            // coll
            if (isPrefixName(ev, "coll:", ptn, begin)) {
                return OgdlCollectionParser.evaluateCollection(ev);
            }
            // constructor
            if (isPrefixName(ev, "cons:", ptn, begin)) {
                return OgdlIntrospectParser.evaluateConstructor(ev);
            }
            break;
        // case 'd':
        // case 'e':
        case 'f':
            // Boolean.FALSE
            if (isLiteral(ev, "false", ptn, begin)) {
                return Boolean.FALSE;
            }
            // field
            if (isPrefixName(ev, "field:", ptn, begin)) {
                return OgdlIntrospectParser.evaluateField(ev);
            }
            // function
            if (isPrefixName(ev, "f:", ptn, begin)) {
                return OgdlFunctionParser.evaluateFunction(ev);
            }
            break;
        // case 'g':
        // case 'h':
        // case 'i':
        // case 'j':
        // case 'k':
        case 'l':
            // list
            if (isPrefixName(ev, "list:", ptn, begin)) {
                return OgdlCollectionParser.evaluateList(ev);
            }
            break;
        case 'm':
            // map
            if (isPrefixName(ev, "map:", ptn, begin)) {
                return OgdlCollectionParser.evaluateMap(ev);
            }
            // method
            if (isPrefixName(ev, "method:", ptn, begin)) {
                return OgdlIntrospectParser.evaluateMethod(ev);
            }
            break;
        case 'n':
            // Null
            if (isLiteral(ev, "null", ptn, begin)) {
                return null;
            }
            // new
            if (isPrefixName(ev, "new:", ptn, begin)) {
                return OgdlIntrospectParser.evaluateNewInstance(ev);
            }
            break;
        // case 'o':
        // case 'p':
        // case 'q':
        case 'r':
            // Reference Operator
            if (isPrefixName(ev, "ref:", ptn, begin)) {
                return OgdlPrimitiveParser.evaluateReference(ev);
            }
            break;
        case 's':
            // set
            if (isPrefixName(ev, "set:", ptn, begin)) {
                return OgdlCollectionParser.evaluateSet(ev);
            }
            // sortedSet
            if (isPrefixName(ev, "sortedSet:", ptn, begin)) {
                return OgdlCollectionParser.evaluateSortedSet(ev);
            }
            // sortedMap
            if (isPrefixName(ev, "sortedMap:", ptn, begin)) {
                return OgdlCollectionParser.evaluateSortedMap(ev);
            }
            break;
        case 't':
            // Boolean.TRUE
            if (isLiteral(ev, "true", ptn, begin)) {
                return Boolean.TRUE;
            }
            break;
        // case 'u':
        // case 'v':
        // case 'w':
        // case 'x':
        // case 'y':
        // case 'z':
        case '{':// Map
            return OgdlCollectionParser.evaluateDefaultMap(ev);
            // case '|':
            // case '}':
        case '~':// Bitwise Complement Operator
            return OgdlUnaryOperatorParser.evaluateBitwiseComplement(ev);
        default:
            break;
        }
        throw new OgdlSyntaxException(ev, "not found expression.");
    }

    /*
     * Collection
     */

    /* コンマ区切りのコレクション書式として解析して返却します。 */
    static Collection evaluateEncloseCollection(OgdlEvent ev) {
        return OgdlCollectionParser.evaluateEncloseCollection(ev);
    }

    /* 開始文字を検証せずに、コンマ区切りのコレクション書式として解析して返却します。 */
    static Collection evaluateCloseCollection(OgdlEvent ev) {
        return OgdlCollectionParser.evaluateCloseCollection(ev);
    }

    /* マップ書式として解析して指定のマップに追加して返却します。 */
    static Map evaluateEncloseMap(OgdlEvent ev) {
        return OgdlCollectionParser.evaluateEncloseMap(ev);
    }

    /* 開始文字を検証せずに、マップ書式として解析して指定のマップに追加して返却します。 */
    static Map evaluateCloseMap(OgdlEvent ev) {
        return OgdlCollectionParser.evaluateCloseMap(ev);
    }

    /* 数値領域（等差数列や時間領域）式を有効とするコンマ区切りのリスト書式として引数のリストに格納します。 */
    static List evaluateEncloseProgressionList(OgdlEvent ev) {
        return (List) OgdlCollectionParser.evaluateEncloseProgressionCollection(ev);
    }

    /* 開始文字を検証せずに、数値領域（等差数列や時間領域）式を有効とするコンマ区切りのリスト書式として解析して引数のリストに格納して返却します。 */
    static List evaluateCloseProgressionList(OgdlEvent ev) {
        return (List) OgdlCollectionParser.evaluateCloseProgressionCollection(ev);
    }

    /*
     * Postfix Expression
     */

    /* 後置式を実行します。 */
    static Object evaluatePostfixExpression(OgdlEvent ev, Object returnValue) {
        final char open = ev.ptn.charAt(ev.off.get());
        switch (open) {
        case '[':
        case '.':
        case '#':
            ev.target = returnValue;
            return OgdlIntrospectParser.evaluateObjectNavigate(ev);
        default:
            break;
        }
        return returnValue;
    }

    /*
     * private
     */

    private static boolean isLiteral(OgdlEvent ev, String literal, String ptn, int begin) {
        if (ptn.startsWith(literal, begin)) {
            ev.off.add(literal.length());
            return true;
        }
        return false;
    }

    /*   */
    private static boolean isPrefixName(OgdlEvent ev, String id, String ptn, int begin) {
        if (ptn.startsWith(id, begin)) {
            ev.off.add(id.length());
            OgdlSyntax.siftSpace(ev);
            if (ev.off.get() >= ev.ptn.length()) {
                throw new OgdlSyntaxException(ev, "offset err.");
            }
            return true;
        }
        return false;
    }

}

==> ogdl-src.0.92.jar: shohaku/ogdl/OgdlParser.java <==
/*
 * shohaku
 * Copyright (C) 2006  tomoya nagatani
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package shohaku.ogdl;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * OGDLの構文解析機能を提供します。<br>
 * このパーザが個別機能を提供する、他のパーザの入口として機能します。
 */
class OgdlParser {

    /*
     * Prefix Expression
     */

    static Object evaluate(OgdlEvent ev) {
        final int open = ev.charAt();
        switch (open) {
        case 0x21: // !
            // Logical Complement Operator
            return OgdlUnaryOperatorParser.evaluateLogicalComplement(ev);
        case 0x25: // %
            // Regex
            return OgdlPrimitiveParser.evaluateRegexPattern(ev);
        case 0x27: // '
            // String
            return OgdlPrimitiveParser.evaluateString(ev);
        case 0x28: // (
            // Set
            return OgdlCollectionParser.evaluateDefaultSet(ev);
        case 0x2B: // +
            // Unary Plus Operator
            return OgdlUnaryOperatorParser.evaluateUnaryPlus(ev);
        case 0x2D: // -
            // Number
            if (ev.hasNext() && OgdlSyntax.isNumHead(ev.charAtBy(1))) {
                return OgdlPrimitiveParser.evaluateNumber(ev);
            }
            // Unary Minus Operator
            return OgdlUnaryOperatorParser.evaluateUnaryMinus(ev);
        case 0x2F: // /
            // Date Time
            return OgdlPrimitiveParser.evaluateDateTime(ev);
        case 0x30: // 0
        case 0x31: // 1
        case 0x32: // 2
        case 0x33: // 3
        case 0x34: // 4
        case 0x35: // 5
        case 0x36: // 6
        case 0x37: // 7
        case 0x38: // 8
        case 0x39: // 9
            // Number
            return OgdlPrimitiveParser.evaluateNumber(ev);
        case 0x40: // @
            // Reference or Class Operator
            if (isLiteral(ev, "@@")) {
                // Class Operator
                return OgdlPrimitiveParser.evaluateClassByExtendName(ev);
            }
            // Reference Operator
            ev.shift();
            return OgdlPrimitiveParser.evaluateReference(ev);
        case 0x42: // B
            ev.shift();
            return OgdlBlockOperatorParser.evaluateOperator(ev);
        case 0x44: // D
            // Date Time
            ev.shift();
            return OgdlPrimitiveParser.evaluateDateTime(ev);
        case 0x46: // F
            // Operator Function
            ev.shift();
            return OgdlOperatorParser.evaluateOperator(ev);
        case 0x49: // I
            // Infinity
            return OgdlPrimitiveParser.evaluateNumber(ev);
        case 0x4E: // N
            // NaN
            return OgdlPrimitiveParser.evaluateNumber(ev);
        case 0x52: // R
            // Regex
            ev.shift();
            return OgdlPrimitiveParser.evaluateRegexPattern(ev);
        case 0x53: // S
            // String
            ev.shift();
            return OgdlPrimitiveParser.evaluateString(ev);
        case 0x55: // U
            // Character
            ev.shift();
            return OgdlPrimitiveParser.evaluateCharacter(ev);
        case 0x5B: // [
            // List
            return OgdlCollectionParser.evaluateDefaultList(ev);
        case 0x60: // `
            // Character
            return OgdlPrimitiveParser.evaluateCharacter(ev);
        case 0x61: // a
            // array
            if (isPrefixName(ev, "array:")) {
                return OgdlCollectionParser.evaluateArray(ev);
            }
            break;
        case 0x63: // c
            // class
            if (isPrefixName(ev, "class:")) {
                return OgdlPrimitiveParser.evaluateClassByExtendName(ev);
            }
            // coll
            if (isPrefixName(ev, "coll:")) {
                return OgdlCollectionParser.evaluateCollection(ev);
            }
            // constructor
            if (isPrefixName(ev, "cons:")) {
                return OgdlIntrospectParser.evaluateConstructor(ev);
            }
            break;
        case 0x66: // f
            // Boolean.FALSE
            if (isLiteral(ev, "false")) {
                return Boolean.FALSE;
            }
            // field
            if (isPrefixName(ev, "field:")) {
                return OgdlIntrospectParser.evaluateField(ev);
            }
            // function
            if (isPrefixName(ev, "f:")) {
                return OgdlFunctionParser.evaluateFunction(ev);
            }
            break;
        case 0x6C: // l
            // list
            if (isPrefixName(ev, "list:")) {
                return OgdlCollectionParser.evaluateList(ev);
            }
            break;
        case 0x6D: // m
            // map
            if (isPrefixName(ev, "map:")) {
                return OgdlCollectionParser.evaluateMap(ev);
            }
            // method
            if (isPrefixName(ev, "method:")) {
                return OgdlIntrospectParser.evaluateMethod(ev);
            }
            break;
        case 0x6E: // n
            // Null
            if (isLiteral(ev, "null")) {
                return null;
            }
            // new
            if (isPrefixName(ev, "new:")) {
                return OgdlIntrospectParser.evaluateNewInstance(ev);
            }
            break;
        case 0x72: // r
            // Reference Operator
            if (isPrefixName(ev, "ref:")) {
                return OgdlPrimitiveParser.evaluateReference(ev);
            }
            break;
        case 0x73: // s
            // set
            if (isPrefixName(ev, "set:")) {
                return OgdlCollectionParser.evaluateSet(ev);
            }
            // sortedSet
            if (isPrefixName(ev, "sortedSet:")) {
                return OgdlCollectionParser.evaluateSortedSet(ev);
            }
            // sortedMap
            if (isPrefixName(ev, "sortedMap:")) {
                return OgdlCollectionParser.evaluateSortedMap(ev);
            }
            break;
        case 0x74: // t
            // Boolean.TRUE
            if (isLiteral(ev, "true")) {
                return Boolean.TRUE;
            }
            break;
        case 0x7B: // {
            // Map
            return OgdlCollectionParser.evaluateDefaultMap(ev);
        case 0x7E: // ~
            // Bitwise Complement Operator
            return OgdlUnaryOperatorParser.evaluateBitwiseComplement(ev);
        default:
            break;
        }
        throw new OgdlSyntaxException(ev, "not find expression.");
    }

    /*
     * Collection
     */

    /* コンマ区切りのコレクション書式として解析して返却します。 */
    static Collection evaluateEncloseCollection(OgdlEvent ev) {
        return OgdlCollectionParser.evaluateEncloseCollection(ev);
    }

    /* 開始文字を検証せずに、コンマ区切りのコレクション書式として解析して返却します。 */
    static Collection evaluateCloseCollection(OgdlEvent ev) {
        return OgdlCollectionParser.evaluateCloseCollection(ev);
    }

    /* マップ書式として解析して指定のマップに追加して返却します。 */
    static Map evaluateEncloseMap(OgdlEvent ev) {
        return OgdlCollectionParser.evaluateEncloseMap(ev);
    }

    /* 開始文字を検証せずに、マップ書式として解析して指定のマップに追加して返却します。 */
    static Map evaluateCloseMap(OgdlEvent ev) {
        return OgdlCollectionParser.evaluateCloseMap(ev);
    }

    /* 数値領域（等差数列や時間領域）式を有効とするコンマ区切りのセット書式として引数のセットに格納します。 */
    static Set evaluateEncloseProgressionSet(OgdlEvent ev) {
        return (Set) OgdlCollectionParser.evaluateEncloseProgressionCollection(ev);
    }

    /* 開始文字を検証せずに、数値領域（等差数列や時間領域）式を有効とするコンマ区切りのセット書式として解析して引数のセットに格納して返却します。 */
    static Set evaluateCloseProgressionSet(OgdlEvent ev) {
        return (Set) OgdlCollectionParser.evaluateCloseProgressionCollection(ev);
    }

    /* 数値領域（等差数列や時間領域）式を有効とするコンマ区切りのリスト書式として引数のリストに格納します。 */
    static List evaluateEncloseProgressionList(OgdlEvent ev) {
        return (List) OgdlCollectionParser.evaluateEncloseProgressionCollection(ev);
    }

    /* 開始文字を検証せずに、数値領域（等差数列や時間領域）式を有効とするコンマ区切りのリスト書式として解析して引数のリストに格納して返却します。 */
    static List evaluateCloseProgressionList(OgdlEvent ev) {
        return (List) OgdlCollectionParser.evaluateCloseProgressionCollection(ev);
    }

    static Class evaluateClass(OgdlEvent ev) {
        return OgdlPrimitiveParser.evaluateClassByExtendName(ev);
    }

    /*
     * Postfix Expression
     */

    /* 後置式を実行します。 */
    static Object evaluatePostfixExpression(OgdlEvent ev, Object returnValue) {
        final int open = ev.charAt();
        switch (open) {
        case '[':
        case '.':
        case '#':
            ev.target = returnValue;
            return OgdlIntrospectParser.evaluateObjectNavigate(ev);
        default:
            break;
        }
        return returnValue;
    }

    /*
     * private
     */

    private static boolean isLiteral(OgdlEvent ev, String literal) {
        if (ev.isPrefix(literal)) {
            ev.shiftBy(literal.length());
            return true;
        }
        return false;
    }

    /*   */
    private static boolean isPrefixName(OgdlEvent ev, String id) {
        if (ev.isPrefix(id)) {
            ev.shiftBy(id.length());
            ev.shiftSpace();
            return true;
        }
        return false;
    }

}
